@@ -3,7 +3,7 @@
 #ifndef CEPH_LIBRBD_WATCHCTX_H
 #define CEPH_LIBRBD_WATCHCTX_H
 
-#include <inttypes.h>
+#include "include/int_types.h"
 
 #include "common/Mutex.h"
 #include "include/buffer.h"
