@@ -1,15 +1,15 @@
-
 // there must be a better way?
 typedef char bool;
 #define false 0
 #define true  1
 
+#include "include/int_types.h"
+
 #include <netinet/in.h>
 #if defined(__linux__)
 #include <linux/types.h>
 #elif defined(__FreeBSD__)
 #include <sys/types.h>
-#include "include/inttypes.h"
 #endif
 #include <string.h>
 #include <fcntl.h>
