@@ -228,7 +228,7 @@ void DataHealthService::service_tick()
     if (ours.latest_avail_percent != last_warned_percent)
       mon->clog.warn()
 	<< "reached concerning levels of available space on local monitor storage"
-	<< " (" << ours.latest_avail_percent << "\% free)\n";
+	<< " (" << ours.latest_avail_percent << "% free)\n";
     last_warned_percent = ours.latest_avail_percent;
   } else {
     last_warned_percent = 0;
