@@ -14,11 +14,11 @@ using namespace std;
 class RGWAccessControlPolicy_SWIFT : public RGWAccessControlPolicy
 {
 public:
-  RGWAccessControlPolicy_SWIFT() {}
+  RGWAccessControlPolicy_SWIFT(CephContext *_cct) : RGWAccessControlPolicy(_cct) {}
   ~RGWAccessControlPolicy_SWIFT() {}
 
-  void add_grants(vector<string>& uids, int perm);
-  bool create(string& id, string& name, string& read_list, string& write_list);
+  void add_grants(RGWRados *store, vector<string>& uids, int perm);
+  bool create(RGWRados *store, string& id, string& name, string& read_list, string& write_list);
   void to_str(string& read, string& write);
 };
 
